from rest_framework import serializers

from tests.models import UserObject


class UserObjectSerializer(serializers.ModelSerializer):
    id = serializers.UUIDField(required=False, read_only=True)
    owner = serializers.UUIDField(required=False, read_only=True)

    class Meta:
        model = UserObject
        fields = '__all__'

    def create(self, validated_data):

        _object = UserObject.objects.create(**validated_data)
        return _object



from generic.helpers import get_current_datetime


def get_log_msg(req, user):
    """
    Returns a log string that contains common data available through views.
    """
    data = req.data.copy()
    if 'password' in data:
        data['password'] = '************'

    templ = "{0}: METHOD: {1}; PATH_INFO: {2}; REMOTE_ADDR: {3}; \
            HTTP_USER_AGENT: {4}; DATA: {5}; USER_ID: {6}"

    msg = templ.format(get_current_datetime(),
                       req.META['req_METHOD'] if 'req_METHOD' in req.META else 'N/A',
                       req.META['PATH_INFO'] if 'PATH_INFO' in req.META else 'N/A',
                       req.META['REMOTE_ADDR'] if 'REMOTE_ADDR' in req.META else 'N/A',
                       req.META['HTTP_USER_AGENT'] if 'HTTP_USER_AGENT' in req.META else 'N/A',
                       data,
                       user.id if user is not None else 'N/A')
    return msg



import string
import uuid
from difflib import SequenceMatcher

from django.utils import timezone


def get_current_datetime():
    """
    Returns the current timestamp taking the timezone into consideration.
    """
    return timezone.now()


def get_uuid1():
    """
    Returns a new uuidv1.
    """
    return uuid.uuid1()


def get_zero_uuid():
    """
    Returns a uuid containing only 0s.
    """
    zero_uuid = uuid.UUID('00000000000000000000000000000000')
    return zero_uuid


def str_to_uuid1(value):
    """
    Converts the given string value into a uuid.
    """
    return uuid.UUID(value)


def are_similar(value_a, value_b):
    """
    Returns the degree of similarity of the two string values a and b.
    """
    sim = SequenceMatcher(None, value_a.lower(), value_b.lower()).ratio()
    return sim


def contains_char_classes(value, char_classes):
    """
    Checks if the given string value contains noc or more classes of characters,
    those classes are: upper case and lower case letters, digits and special
    characters.
    """
    found = 0

    if any(c in value for c in string.ascii_lowercase):
        found += 1
    if any(c in value for c in string.ascii_uppercase):
        found += 1
    if any(c in value for c in string.digits):
        found += 1
    if any(c in value for c in string.punctuation):
        found += 1

    if found >= char_classes:
        return True

    return False


def isclose(a, b, rel_tol=1e-09, abs_tol=0.0):
    """
    Checks if two floating point numbers are almost equal.
    """
    return abs(a - b) <= max(rel_tol * max(abs(a), abs(b)), abs_tol)



import re

from django.core.exceptions import ObjectDoesNotExist

from generic.exceptions import ObjectNotFoundError
from generic.helpers import contains_char_classes


class ValidatorMixin:
    """
    Provides validators for generic repositories.
    """

    def is_unique(self, instance, field):
        """
        :arg: instance: Instance that should be validated
        :arg: field: Field that should be checked for uniqueness
        :returns: True if the value stored in field is unique; False otherwise

        Checks the uniqueness of the value stored in field.
        Cannot be used as an argument validator.
        """
        try:
            value = getattr(instance, field)
        except KeyError:
            return True

        if value is None:
            return True

        kwargs = {field: str(value)}
        pk = getattr(instance, 'id')
        conflict = self.model_class.objects.exclude(id=pk).filter(**kwargs).count()

        if conflict is 0:
            return True

        return False

    @staticmethod
    def is_none(instance, field):
        """
        :arg: instance: Instance that should be validated
        :arg: field: field: Field that should be checked for None
        :returns: True if the value stored in field is None; False otherwise

        Checks if the value stored in field is None.
        Can be used as an argument validator.
        """
        try:
            if type(instance) is dict:
                value = instance[field]
            else:
                value = getattr(instance, field)

        except KeyError:
            return True

        if value is None:
            return True

        return False

    @staticmethod
    def is_too_long(instance, field, length):
        """
        :arg: instance: Instance that should be validated
        :arg: field: field: Field that should be checked for length
        :returns: True if the value stored in field if longer than length;
        False otherwise

        Checks if the value stored in field is longer then max_length.
        Only applicable to CharFields.
        Can be used as an argument validator.
        """
        try:
            if type(instance) is dict:
                value = instance[field]
            else:
                value = getattr(instance, field)

        except KeyError:
            return False

        if value is None:
            return False

        if len(value) > length:
            return True

        return False

    @staticmethod
    def is_too_short(instance, field, length):
        """
        :arg: instance: Instance that should be validated
        :arg: field: field: Field that should be checked for length
        :returns: True if the value stored in field if shorter than length;
        False otherwise

        Checks if the value stored in field is shorter then length.
        Only applicable to CharFields.
        Can be used as an argument validator.
        """
        try:
            if type(instance) is dict:
                value = instance[field]
            else:
                value = getattr(instance, field)

        except KeyError:
            return False

        if value is None:
            return False

        if len(value) < length:
            return True

        return False

    @staticmethod
    def contains_arg(instance, field):
        """
        :arg: instance: Instance that should be validated
        :arg: field: field: Field that should be checked for presence
        :returns: True if the argument is present; False otherwise

        Checks if field is present in the args dictionary or instance.
        Can be used as an argument validator.
        """
        try:
            if type(instance) is dict:
                instance[field]
            else:
                getattr(instance, field)

        except KeyError:
            return False

        return True

    @staticmethod
    def contains_char_classes(instance, field, char_classes):
        """
        :arg: instance: Instance that should be validated
        :arg: field: Field that should be checked for character classes
        :arg: char_classes: Number of character classes that must be present
        :returns: True if the value stored in field contains sufficient
        character classes, False otherwise

        Checks if the value stored in field contains the demanded number of
        different character classes.
        Can be used as an argument validator.
        """
        try:
            if type(instance) is dict:
                value = instance[field]
            else:
                value = getattr(instance, field)

        except KeyError:
            return True

        if value is None:
            return True

        if contains_char_classes(value, char_classes):
            return True

        return False

    @staticmethod
    def is_valid_email(instance, field):
        """
        :arg: instance: Instance that should be validated
        :arg: field: Field that should be checked for a valid email address
        :returns: True if the value stored in field is a valid email address;
        False otherwise

        Checks if the value stored in field is a valid email address.
        Can be used as an argument validator.
        """
        try:
            if type(instance) is dict:
                value = instance[field]
            else:
                value = getattr(instance, field)

        except KeyError:
            return True

        if value is None:
            return True

        reg_ex = r"[^@]+@[^@]+\.[^@]+"
        if re.match(reg_ex, value):
            return True

        return False


class GenericRepository(ValidatorMixin):
    """
    Provides a generic database abstraction layer for all model classes.
    """

    def __init__(self, model_class):
        """
        :arg: model_class: Class of the model that this repository should
        work on.

        Constructor that initializes the repository with a specific
        model class.
        """
        self.model_class = model_class

    def find_by_user(self, user=None):
        """
        :raises: NotImplementedError
        """
        raise NotImplementedError("""GenericRepository does not provide
                                  function find_by_user.""")

    def find_by_id(self, pk, user=None):
        """
        :arg: pk: Primary key of the instance that should be returned
        :returns: Queryset representing the instance with id=pk

        Provides generic get by id function. Returns queryset representing
        exactly one instance or an empty queryset.
        """
        return self.model_class.objects.filter(id=pk)

    def get_by_id(self, pk, user=None):
        """
        :arg: pk: Primary key of the instance that should be returned
        :returns: Requested instance with id=pk
        :raises: ObjectNotFoundError if the requested object was not found

        Provides generic get by id function. Returns exactly one instance or
        raises error.
        """
        try:
            return self.model_class.objects.get(id=pk)
        except ObjectDoesNotExist:
            raise ObjectNotFoundError('Object not found.',
                                      modelclass=self.model_class)

    def list_all(self, user=None):
        """
        :returns: Queryset of all instances

        Provides generic list all function.
        """
        return self.model_class.objects.all()

    def delete_by_id(self, pk, user=None):
        """
        :arg: pk: Primary key of the instance that should be deleted
        :returns: True if the instance was deleted
        :raises: ObjectNotFoundError if the requested object was not found

        Provides generic delete function.
        """
        try:
            self.model_class.objects.get(id=pk).delete()
            return True
        except ObjectDoesNotExist:
            raise ObjectNotFoundError('Object not found.',
                                      modelclass=self.model_class)

    @staticmethod
    def persist(instance):
        """
        :arg: instance: Instance that should be persisted.
        :returns: The persisted instance

        Provides generic persist function.
        """
        return instance.save()


class OwnershipRepository(ValidatorMixin):
    """
    Provides a generic database abstraction layer for all model classes
    that have an owner field, writing and reading of these model classes is
    limited to the owner. At time of creation the ownership information is
    automatically inserted.
    """

    def __init__(self, model_class):
        """
        :arg: model_class: Class of the model that this repository should
        work on.

        Constructor that initializes the repository with a specific
        model class.
        """
        self.model_class = model_class

    def find_by_user(self, user):
        """
        :arg: user: Requesting user
        :returns: Queryset representing all instances owned by user

        Provides generic find by user function.
        """
        if hasattr(self.model_class, 'owner'):
            return self.model_class.objects.filter(owner=user)

        return self.model_class.objects.filter(id=user.id)

    def find_by_id(self, pk, user):
        """
        :arg: pk: Primary key of the instance that should be returned
        :arg: user: Requesting user
        :returns: Queryset representing the instance owned by user with id=pk

        Provides generic get by id function. Only matches instances that are
        owned by the user. If there is no owner field, id is used for
        determining ownership instead. Returns queryset representing exactly
        one instance or an empty queryset.
        """
        if hasattr(self.model_class, 'owner'):
            return self.model_class.objects.filter(owner=user).filter(id=pk)

        return self.model_class.objects.filter(id=user.id).filter(id=pk)

    def get_by_id(self, pk, user):
        """
        :arg: pk: Primary key of the instance that should be returned
        :arg: user: Requesting user
        :returns: Requested instance owned by user with id=pk
        :raises: ObjectNotFoundError if the requested object was either not
        found or not owned by user

        Provides generic get by id function. Only matches instances that are
        owned by the user. If there is no owner field, id is used for
        determining ownership instead. Returns exactly one instance or
        raises error.
        """
        if hasattr(self.model_class, 'owner'):
            try:
                return self.model_class.objects.filter(owner=user).get(id=pk)
            except ObjectDoesNotExist:
                raise ObjectNotFoundError('Object not found.',
                                          modelclass=self.model_class)

        try:
            return self.model_class.objects.filter(id=user.id).get(id=pk)
        except (ObjectDoesNotExist, AttributeError):
            raise ObjectNotFoundError('Object not found.',
                                      modelclass=self.model_class)

    def list_all(self, user):
        """
        :arg: user: Requesting user
        :returns: Queryset of instances owned by the user

        Provides generic list all owned by user function. If there is no owner
        field, id is used for determining ownership instead.
        """
        if hasattr(self.model_class, 'owner'):
            return self.model_class.objects.filter(owner=user)

        return self.model_class.objects.filter(id=user.id)

    def delete_by_id(self, pk, user):
        """
        :arg: pk: Primary key of the instance that should be deleted
        :arg: user: Requesting user
        :returns: Instance that was deleted
        :raises: ObjectNotFoundError if the requested object was either not
        found or not owned by user

        Provides generic delete function. Only matches instances that are
        owned by the user. If there is no owner field, id is used for
        determining ownership instead.
        """
        if hasattr(self.model_class, 'owner'):
            try:
                return self.model_class.objects.filter(owner=user).get(id=pk).delete()
            except ObjectDoesNotExist:
                raise ObjectNotFoundError('Object not found.',
                                          modelclass=self.model_class)

        try:
            return self.model_class.objects.filter(id=user.id).get(id=pk).delete()
        except (ObjectDoesNotExist, AttributeError):
            raise ObjectNotFoundError('Object not found.',
                                      modelclass=self.model_class)

    @staticmethod
    def persist(instance):
        """
        :arg: instance: Instance that should be persisted.
        :returns: The persisted instance

        Provides generic persist function.
        """
        return instance.save()



from django.core.exceptions import ImproperlyConfigured
from django.db.models.query import QuerySet

from generic.exceptions import ValidationError


def unique_field(repository, field):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Checks if a field is unique.
    """

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if not repository.is_unique(instance, field):
                raise ValidationError('This value has alredy been used.',
                                      field=field)
            return True

    return _checker


def required_field(repository, field):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Checks if a field is present.
    """

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if repository.is_none(instance, field):
                raise ValidationError('Mandatory value was not provided.',
                                      field=field)
            return True

    return _checker


def max_length(repository, field, length):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :arg: length: Maximal length for field
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Checks if the max length of a CharField is exceeded.
    """

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if repository.is_too_long(instance, field, length):
                raise ValidationError(("Maximal length of {0} chars "
                                      "exceeded.").format(length),
                                      field=field)
            return True

    return _checker


def min_length(repository, field, length):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :arg: length: Maximal length for field
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Checks if the min length of a CharField is is met.
    """

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if repository.is_too_short(instance, field, length):
                raise ValidationError(("Minimal length of {0} characters "
                                      "not met.").format(length),
                                      field=field)

            return True

    return _checker


def read_only_field(repository, field):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Ensures that read only fields are not written to.
    """

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if repository.contains_arg(instance, field):
                raise ValidationError('Attempt to write read only field.',
                                      field=field)
            return True

    return _checker


def min_char_classes(repository, field, char_classes):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :arg: char_classes: Number of different character classes need to be
    present; can be lower/upper case letters, digits or punctuation
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Checks if field comprises sufficient character classes.
    """
    if char_classes < 0 or char_classes > 4:
        raise ImproperlyConfigured(("Number of character classes "
                                   "must be  in range 0-4."))

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if not repository.contains_char_classes(instance, field, char_classes):
                raise ValidationError(("Field must contain {0} of the "
                                      "following characters: upper case letters, "
                                      "lower case letters, digits and special "
                                      "characters.").format(char_classes),
                                      field='password')

            return True

    return _checker


def email_field(repository, field):
    """
    :arg: repository: Repository that should be used to perform validation
    :arg: field: Field to be validated
    :returns: True is validation was successful
    :raises: ValidationError if the validation failed

    Checks is a user's email address is valid
    """

    def _checker(instance):
        if isinstance(instance, QuerySet):
            return all(_checker(i) for i in instance)
        else:
            if not repository.is_valid_email(instance, field):
                raise ValidationError('This email address is invalid.', field='email')
            return True

    return _checker


def validate_many(*validators):
    """
    :arg: *validators: List of validators to be applied

    Allows to apply multiple validators.
    """

    def _checker(instance):
        return all(validator(instance) for validator in validators)

    return _checker



import uuid

from django.test import TransactionTestCase
from django.contrib.auth.hashers import check_password
from rest_framework.test import APIClient

from generic.exceptions import ObjectNotFoundError, ValidationError
from generic.helpers import str_to_uuid1
from tests.models import User, Object, UserObject, UserObjectPw
from generic.factories import generic_factory
from generic.validators import unique_field, validate_many
from generic.repositories import (GenericRepository, OwnershipRepository)
from generic.services import (generic_retrieve_single_service,
                              generic_retrieve_all_service,
                              generic_retrieve_all_by_owner_service,
                              generic_delete_service,
                              generic_update_service,
                              generic_create_service)


class GenericFactoryTest(TransactionTestCase):
    def test_factory(self):
        name = 'new object'
        kwargs = {'name': name}
        o = generic_factory(model_class=Object, **kwargs)
        self.assertEqual(Object, type(o))
        self.assertEqual(type(uuid.uuid1()), type(o.id))
        self.assertEqual(name, o.name)


class GenericRepositoryTest(TransactionTestCase):
    fixtures = ['init.json']

    def test_find_by_user(self):
        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        with self.assertRaises(NotImplementedError):
            repo.find_by_user(u)

        repo = GenericRepository(Object)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        with self.assertRaises(NotImplementedError):
            repo.find_by_user(u)

    def test_find_by_id(self):
        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        o1 = UserObject.objects.get(id=o_id)
        o2 = list(repo.find_by_id(pk=o_id, user=u))[0]
        self.assertEqual(o1, o2)

        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('87eca58a-5f71-11e8-b471-f40f2434c1ce')
        length = len(list(repo.find_by_id(pk=o_id, user=u)))
        self.assertEqual(1, length)

        repo = GenericRepository(Object)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        o2 = list(repo.find_by_id(pk=o_id, user=u))
        self.assertEqual(1, len(o2))

    def test_get_by_id(self):
        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        o1 = UserObject.objects.get(id=o_id)
        o2 = repo.get_by_id(pk=o_id, user=u)
        self.assertEqual(o1, o2)

        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('87eca58a-5f71-11e8-b471-f40f2434c1ce')
        instance =  repo.get_by_id(pk=o_id, user=u)
        self.assertEqual(o_id, instance.id)

        repo = GenericRepository(Object)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        instance = repo.get_by_id(pk=o_id, user=u)
        self.assertEqual(o_id, instance.id)

        repo = GenericRepository(Object)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('e488a5ba-5f78-11e8-b89d-f40f2434c1ce')
        with self.assertRaises(ObjectNotFoundError):
            repo.get_by_id(pk=o_id, user=u)

    def test_get_by_user_and_id(self):
        repo = OwnershipRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        o1 = UserObject.objects.get(id=o_id)
        o2 = repo.get_by_id(pk=o_id, user=u)
        self.assertEqual(o1, o2)

        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('87eca58a-5f71-11e8-b471-f40f2434c1ce')
        instance = repo.get_by_id(pk=o_id, user=u)
        self.assertEqual(o_id, instance.id)

        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('fd88db5e-5f82-11e8-a7f9-f40f2434c1ce')
        with self.assertRaises(ObjectNotFoundError):
            repo.get_by_id(pk=o_id, user=u)

    def test_list_all(self):
        repo = GenericRepository(UserObject)
        l1 = len(list(UserObject.objects.all()))
        l2 = len(list(repo.list_all()))
        self.assertEqual(l1, l2)

    def test_persist(self):
        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        l1 = len(list(UserObject.objects.filter(owner=u)))
        instance = UserObject(name='A new user object', owner=u)
        repo.persist(instance)
        l2 = len(list(UserObject.objects.filter(owner=u)))
        self.assertEqual(l1, (l2 - 1))

    def test_delete_by_id(self):
        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        l1 = len(list(UserObject.objects.filter(owner=u)))
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        repo.delete_by_id(pk=o_id, user=u)
        l2 = len(list(UserObject.objects.filter(owner=u)))
        self.assertEqual(l1, (l2 + 1))

        repo = GenericRepository(UserObject)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('87eca58a-5f71-11e8-b471-f40f2434c1ce')
        deleted = repo.delete_by_id(pk=o_id, user=u)
        self.assertEqual(True, deleted)

        repo = GenericRepository(Object)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        deleted = repo.delete_by_id(pk=o_id, user=u)
        self.assertEqual(True, deleted)

    def test_is_unique(self):
        repo = GenericRepository(Object)
        o_id = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        obj = Object.objects.get(id=o_id)
        obj.name = 'Another Object'
        unique = repo.is_unique(obj, 'name')
        self.assertEqual(False, unique)

        repo = GenericRepository(Object)
        o_id = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        obj = Object.objects.get(id=o_id)
        unique = repo.is_unique(obj, 'name')
        self.assertEqual(True, unique)

        repo = GenericRepository(Object)
        o_id = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        obj = Object.objects.get(id=o_id)
        unique = repo.is_unique(obj, 'name')
        self.assertEqual(True, unique)

        repo = GenericRepository(Object)
        o_id = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        obj = Object.objects.get(id=o_id)
        unique = repo.is_unique(obj, 'name')
        self.assertEqual(True, unique)

    def test_is_too_short(self):
        repo = GenericRepository(Object)
        pk = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        instance = Object.objects.get(id=pk)
        too_short = repo.is_too_short(instance, 'name', 3)
        self.assertEqual(False, too_short)

        repo = GenericRepository(Object)
        pk = str_to_uuid1('4343f01c-5f71-11e8-b73c-f40f2434c1ce')
        instance = Object.objects.get(id=pk)
        too_short = repo.is_too_short(instance, 'name', 15)
        self.assertEqual(True, too_short)

    def test_contains_char_classes(self):
        repo = GenericRepository(Object)
        pk = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        classes = repo.contains_char_classes(instance, 'name', 1)
        self.assertEqual(True, classes)

        repo = GenericRepository(Object)
        pk = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        classes = repo.contains_char_classes(instance, 'name', 2)
        self.assertEqual(True, classes)

        repo = GenericRepository(Object)
        pk = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        classes = repo.contains_char_classes(instance, 'name', 3)
        self.assertEqual(False, classes)

        repo = GenericRepository(Object)
        pk = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        instance.name = 'aA#1'
        classes = repo.contains_char_classes(instance, 'name', 4)
        self.assertEqual(True, classes)

    def test_is_valid_email(self):
        repo = GenericRepository(User)
        pk = str_to_uuid1('53fbd124-1c69-11e8-941a-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        valid = repo.is_valid_email(instance, 'email')
        self.assertEqual(True, valid)

        repo = GenericRepository(User)
        pk = str_to_uuid1('53fbd124-1c69-11e8-941a-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        instance.email = 'not valid'
        valid = repo.is_valid_email(instance, 'email')
        self.assertEqual(False, valid)

        repo = GenericRepository(User)
        pk = str_to_uuid1('53fbd124-1c69-11e8-941a-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        instance.email = None
        valid = repo.is_valid_email(instance, 'email')
        self.assertEqual(True, valid)

        repo = GenericRepository(User)
        pk = str_to_uuid1('53fbd124-1c69-11e8-941a-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        instance.email = 'john.doeexample.com'
        valid = repo.is_valid_email(instance, 'email')
        self.assertEqual(False, valid)

        repo = GenericRepository(User)
        pk = str_to_uuid1('53fbd124-1c69-11e8-941a-f40f2434c1ce')
        instance = repo.get_by_id(pk)
        instance.email = 'john@example.com'
        valid = repo.is_valid_email(instance, 'email')
        self.assertEqual(True, valid)


class GenericValidatorTest(TransactionTestCase):
    fixtures = ['init.json']

    def test_is_value_unique(self):
        repo = GenericRepository(Object)
        o_id = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        o = Object.objects.get(id=o_id)

        validator = unique_field(repo, 'name')
        self.assertEqual(True, validator(o))

        repo = GenericRepository(Object)
        o = Object(name='Object')

        validator = unique_field(repo, 'name')
        with self.assertRaises(ValidationError):
            validator(o)

        repo = GenericRepository(Object)
        os = Object.objects.all()

        validator = unique_field(repo, 'name')
        self.assertEqual(True, validator(os))

    def test_validate_many(self):
        repo = GenericRepository(UserObject)
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        o = UserObject.objects.get(id=o_id)

        validators = (unique_field(repo, 'name'),
                      unique_field(repo, 'owner'))
        checker = validate_many(*validators)
        self.assertEqual(True, checker(o))


class GenericServiceTest(TransactionTestCase):
    fixtures = ['init.json']

    def test_retrieve_single_service(self):
        repo = GenericRepository(Object)
        retrieve = generic_retrieve_single_service(repo)
        o_id = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        instance = retrieve(pk=o_id, requesting_user=None)
        self.assertEqual(o_id, instance.id)

    def test_retrieve_all(self):
        repo = GenericRepository(Object)
        retrieve = generic_retrieve_all_service(repo)
        l = len(retrieve())
        self.assertEqual(2, l)

    def test_retrieve_all_by_owner_service(self):
        repo = GenericRepository(Object)
        retrieve = generic_retrieve_all_by_owner_service(repo)
        with self.assertRaises(NotImplementedError):
            retrieve(requesting_user=None)

    def test_delete_service(self):
        repo = GenericRepository(Object)
        delete = generic_delete_service(repo)
        o_id = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        instance = delete(pk=o_id, requesting_user=u)
        self.assertEqual(o_id, instance.id)

        repo = GenericRepository(UserObject)
        delete = generic_delete_service(repo)
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        delete(pk=o_id, requesting_user=u)
        o_l = len(list(UserObject.objects.filter(id=o_id)))
        self.assertEqual(0, o_l)

        repo = GenericRepository(UserObject)
        delete = generic_delete_service(repo)
        o_id = str_to_uuid1('87eca58a-5f71-11e8-b471-f40f2434c1ce')
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        instance = delete(pk=o_id, requesting_user=u)
        self.assertEqual(o_id, instance.id)

    def test_update_service(self):
        repo = GenericRepository(Object)
        validator = (unique_field(repo, 'name'),)
        update = generic_update_service(repo, validate_many(*validator))
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('6ef912ae-5f71-11e8-bc5f-f40f2434c1ce')
        name = 'NewNameForOldObject'
        password = 'newSecure#1Pass'

        instance = update(pk=o_id, requesting_user=u, name=name, password=password)
        self.assertEqual(name, instance.name)
        self.assertEqual(True, check_password(password, instance.password))

        repo = GenericRepository(UserObject)
        validator = (unique_field(repo, 'name'),)
        update = generic_update_service(repo, validate_many(*validator))
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('83695382-5f71-11e8-9d7e-f40f2434c1ce')
        name = 'NewNameForOldObject'

        update(pk=o_id, requesting_user=u, name=name)
        o = UserObject.objects.get(id=o_id)
        self.assertEqual(o_id, o.id)
        self.assertEqual(name, o.name)

        repo = GenericRepository(UserObjectPw)
        validator = (unique_field(repo, 'name'),)
        update = generic_update_service(repo, validate_many(*validator))
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o_id = str_to_uuid1('603378ab-5f85-11e8-8ef9-f40f2434c1ce')
        name = 'NewNameForOldObject'
        password = 'NewSecurePassword#123'

        update(pk=o_id, requesting_user=u, name=name, password=password)
        o = UserObjectPw.objects.get(id=o_id)
        self.assertEqual(o_id, o.id)
        self.assertEqual(name, o.name)
        self.assertEqual(True, check_password(password, o.password))

    def test_create_service(self):
        repo = GenericRepository(Object)
        validator = (unique_field(repo, 'name'),)
        create = generic_create_service(repo,
                                        validate_many(*validator),
                                        generic_factory)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        password = 'New123Password'
        o1 = create(requesting_user=u, name='VeryNewObject', password=password)
        o2 = Object.objects.get(id=o1.id)
        self.assertEqual(o1, o2)
        self.assertEqual(True, check_password(password, o1.password))
        self.assertEqual(True, check_password(password, o2.password))

        repo = GenericRepository(UserObject)
        validator = (unique_field(repo, 'name'),)
        create = generic_create_service(repo,
                                        validate_many(*validator),
                                        generic_factory)
        user_id = str_to_uuid1('3f44d78a-1c69-11e8-a078-f40f2434c1ce')
        u = User.objects.get(id=user_id)
        o1 = create(requesting_user=u, name='VeryNewObject')
        o2 = UserObject.objects.get(id=o1.id)
        self.assertEqual(o1, o2)


class GenericViewTest(TransactionTestCase):
    fixtures = ['init.json']

    def test_create_view(self):
        user = User.objects.get(username='lisamorgan')
        client = APIClient()
        client.force_authenticate(user=user)

        l1 = len(list(UserObject.objects.all()))
        request = client.post('/objects.json', {'name': 'new name'}, format='json')
        l2 = len(list(UserObject.objects.all()))
        self.assertEqual(201, request.status_code)
        self.assertEqual(l1, (l2 - 1))

    def test_list_view(self):
        user = User.objects.get(username='lisamorgan')
        client = APIClient()
        client.force_authenticate(user=user)

        request = client.get('/objects.json', format='json')
        self.assertEqual(200, request.status_code)

    def test_retrieve_view(self):
        user = User.objects.get(username='lisamorgan')
        client = APIClient()
        client.force_authenticate(user=user)

        request = client.get('/objects/83695382-5f71-11e8-9d7e-f40f2434c1ce.json',
                             format='json')
        self.assertEqual(200, request.status_code)

    def test_destroy_view(self):
        user = User.objects.get(username='lisamorgan')
        client = APIClient()
        client.force_authenticate(user=user)

        l1 = len(list(UserObject.objects.all()))
        request = client.delete('/objects/83695382-5f71-11e8-9d7e-f40f2434c1ce.json',
                                format='json')
        l2 = len(list(UserObject.objects.all()))
        self.assertEqual(200, request.status_code)
        self.assertEqual(l1, (l2 + 1))

    def test_partial_update_view(self):
        user = User.objects.get(username='lisamorgan')
        client = APIClient()
        client.force_authenticate(user=user)

        request = client.patch('/objects/83695382-5f71-11e8-9d7e-f40f2434c1ce.json',
                               {'name': 'new name2'}, format='json')
        self.assertEqual(200, request.status_code)



from datetime import timedelta

SECRET_KEY = '2^mza*qpug3+htv7jxecatc0w&rluw!b#2cf9r*+&3fj8a2i66'
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'tests',
    'generic',
]
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'django-generic-rest.db',
    }
}
AUTH_USER_MODEL = 'tests.User'
USE_TZ = True
ERROR_KEY = 'Error'
ROOT_URLCONF = 'tests.urls'

# Generic Token Authentication
TOKEN_LENGTH = 64
TOKEN_EXPIRY = timedelta(hours=1)
REFRESH_TOKEN_LENGTH = 128
REFRESH_TOKEN_EXPIRY = timedelta(days=1)



from rest_framework.urlpatterns import format_suffix_patterns
from rest_framework import routers

from . import views

app_name = 'tests'

router = routers.SimpleRouter()
router.register(r'objects', views.ObjectViewSet, base_name='objects')
urlpatterns = router.urls

urlpatterns = format_suffix_patterns(
    urlpatterns, suffix_required=True, allowed=['json'])
